/*
 * Driver for the TAS5733L Audio Amplifier driver
 * 
 * Copyright (C) 2018 Harman Connected services.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * version 2 as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 */
#include<linux/delay.h>

#define CFG_META_DELAY  254
#define CFG_META_BURST  253

#define DEBUG_ENABLE 1
typedef unsigned char cfg_u8;

typedef union {
 struct {
	cfg_u8 reg;
	cfg_u8 value;
 };
 struct {
	cfg_u8 command;
	cfg_u8 param;
 };
}cfg_reg;

/* AMP init register */

#define CLK_CTRL	0x00
#define DEVICE_ID	0x01
#define ERROR_STATUS	0x02
#define SYSTEM_CTRL	0x05
#define SOFT_MUTE	0x06
#define MASTER_VOL	0x07
#define CH1_VOL		0x08
#define CH2_VOL		0x09
#define CH3_VOL		0x0A
#define MOD_LIMIT	0x10
#define OSC_TRIM	0x1B

#define SYSTEM_DEF_VAL	0x00
#define DEF_MOD_VAL	0x07
#define DEF_VAL_GAIN	0xC000
#define SOFT_MUTE_VAL	0x03
#define LOW_GAIN	0xE003

/* PPC3 default intialisation */

cfg_reg registers[] = {

    { 0x1b, 0x00 },
    { 0x05, 0x00 },
    { 0x06, 0x00 },
    { 0x10, 0x07 },
	{ CFG_META_BURST, 3},
    { 0x07, 0x00 },
    { 0xc0, 0x00 },
    { 0x03, 0xa0 },
    { 0x04, 0x05 },
   { CFG_META_BURST, 3 },
    { 0x08, 0x00 },
    { 0xc0, 0x00 },
    { CFG_META_BURST, 3 },
    { 0x09, 0x00 },
    { 0xc0, 0x00 },
    { CFG_META_BURST, 3 },
    { 0x0a, 0x00 },
    { 0xc0, 0x00 },
    { 0x0e, 0xf0 },
    { 0x10, 0x07 },
    { 0x11, 0xb8 },
    { 0x12, 0x60 },
    { 0x13, 0xa0 },
    { 0x14, 0x48 },
    { 0x19, 0x30 },
    { 0x1a, 0x68 },
    { 0x1c, 0x57 },
    { CFG_META_BURST, 5 },
    { 0x20, 0x00 },
    { 0x89, 0x77 },
    { 0x72, 0x00 },
    { CFG_META_BURST, 5 },
    { 0x25, 0x02 },
    { 0x02, 0x13 },
    { 0x45, 0x00 },
    { CFG_META_BURST, 21 },
    { 0x26, 0x00 },
    { 0x7f, 0x2a },
    { 0x95, 0x03 },
    { 0x01, 0xaa },
    { 0xd6, 0x00 },
    { 0x7f, 0x2a },
    { 0x95, 0x00 },
    { 0xfe, 0x54 },
    { 0x77, 0x03 },
    { 0x81, 0xaa },
    { 0x24, 0x00 },
	{ CFG_META_DELAY, 100 },
    { CFG_META_BURST, 21 },
    { 0x27, 0x00 },
    { 0x83, 0xf4 },
    { 0x57, 0x03 },
    { 0x02, 0xaa },
    { 0xbc, 0x00 },
    { 0x79, 0x64 },
    { 0x7b, 0x00 },
    { 0xfd, 0x55 },
    { 0x44, 0x03 },
    { 0x82, 0xa7 },
    { 0x2e, 0x00 },
	{ CFG_META_DELAY, 100 },
    { CFG_META_BURST, 21 },
    { 0x28, 0x00 },
    { 0x7f, 0x0c },
    { 0xdf, 0x03 },
    { 0x03, 0xe1 },
    { 0xc4, 0x00 },
    { 0x7d, 0x24 },
    { 0x4b, 0x00 },
    { 0xfc, 0x1e },
    { 0x3c, 0x03 },
    { 0x83, 0xce },
    { 0xd6, 0x00 },
	{ CFG_META_DELAY, 100 },
    { CFG_META_BURST, 21 },
    { 0x29, 0x00 },
    { 0x76, 0xd2 },
    { 0x06, 0x03 },
    { 0x17, 0x1d },
    { 0x76, 0x00 },
    { 0x72, 0x3e },
    { 0x84, 0x00 },
    { 0xe8, 0xe2 },
    { 0x8a, 0x03 },
    { 0x96, 0xef },
    { 0x76, 0x00 },
	{ CFG_META_DELAY, 100 },
    { CFG_META_BURST, 21 },
    { 0x2a, 0x00 },
    { 0x79, 0x22 },
    { 0x7d, 0x03 },
    { 0x2a, 0x5d },
    { 0xe5, 0x00 },
    { 0x67, 0x7e },
    { 0x17, 0x00 },
    { 0xd5, 0xa2 },
    { 0x1b, 0x03 },
    { 0x9f, 0x5f },
    { 0x6b, 0x00 },
	{ CFG_META_DELAY, 100 },
    { CFG_META_BURST, 21 },
    { 0x2b, 0x00 },
    { 0x79, 0x42 },
    { 0x12, 0x03 },
    { 0x17, 0x61 },
    { 0xe0, 0x00 },
    { 0x70, 0x56 },
    { 0xed, 0x00 },
    { 0xe8, 0x9e },
    { 0x20, 0x03 },
    { 0x96, 0x67 },
    { 0x01, 0x00 },
	{ CFG_META_DELAY, 100 },
    { CFG_META_BURST, 21 },
    { 0x2c, 0x00 },
    { 0x7c, 0x61 },
    { 0x61, 0x03 },
    { 0x33, 0x49 },
    { 0x22, 0x00 },
    { 0x70, 0x00 },
    { 0xdc, 0x00 },
    { 0xcc, 0xb6 },
    { 0xde, 0x03 },
    { 0x93, 0x9d },
    { 0xc3, 0x00 },
	{ CFG_META_DELAY, 100 },
    { CFG_META_BURST, 21 },
    { 0x2d, 0x00 },
    { 0x76, 0xf2 },
    { 0xe1, 0x03 },
    { 0x25, 0x01 },
    { 0xb9, 0x00 },
    { 0x68, 0x55 },
    { 0xba, 0x00 },
    { 0xda, 0xfe },
    { 0x47, 0x03 },
    { 0xa0, 0xb7 },
    { 0x66, 0x00 },
	{ CFG_META_DELAY, 100 },
    { CFG_META_BURST, 21 },
    { 0x2e, 0x00 },
    { 0x80, 0x00 },
    { 0x00, 0x00 },
    { 0x00, 0x00 },
    { 0x00, 0x00 },
    { 0x00, 0x00 },
    { 0x00, 0x00 },
    { 0x00, 0x00 },
    { 0x00, 0x00 },
    { 0x00, 0x00 },
    { 0x00, 0x00 },
	{ CFG_META_DELAY, 100 },
    { CFG_META_BURST, 21 },
    { 0x2f, 0x00 },
    { 0x4d, 0xe0 },
    { 0xc7, 0x03 },
    { 0x94, 0xe7 },
    { 0x19, 0x00 },
    { 0x29, 0x26 },
    { 0x6f, 0x00 },
    { 0xc3, 0x17 },
    { 0x77, 0x03 },
    { 0xb0, 0xfa },
    { 0x3a, 0x00 },
	{ CFG_META_DELAY, 100 },
    { CFG_META_BURST, 21 },
    { 0x30, 0x00 },
    { 0x7f, 0x2a },
    { 0x95, 0x03 },
    { 0x01, 0xaa },
    { 0xd6, 0x00 },
    { 0x7f, 0x2a },
    { 0x95, 0x00 },
    { 0xfe, 0x54 },
    { 0x77, 0x03 },
    { 0x81, 0xaa },
    { 0x24, 0x00 },
	{ CFG_META_DELAY, 100 },
    { CFG_META_BURST, 21 },
    { 0x31, 0x00 },
    { 0x83, 0xf4 },
    { 0x57, 0x03 },
    { 0x02, 0xaa },
    { 0xbc, 0x00 },
    { 0x79, 0x64 },
    { 0x7b, 0x00 },
    { 0xfd, 0x55 },
    { 0x44, 0x03 },
    { 0x82, 0xa7 },
    { 0x2e, 0x00 },
	{ CFG_META_DELAY, 100 },
    { CFG_META_BURST, 21 },
    { 0x32, 0x00 },
    { 0x7f, 0x0c },
    { 0xdf, 0x03 },
    { 0x03, 0xe1 },
    { 0xc4, 0x00 },
    { 0x7d, 0x24 },
    { 0x4b, 0x00 },
    { 0xfc, 0x1e },
    { 0x3c, 0x03 },
    { 0x83, 0xce },
    { 0xd6, 0x00 },
	{ CFG_META_DELAY, 100 },
    { CFG_META_BURST, 21 },
    { 0x33, 0x00 },
    { 0x76, 0xd2 },
    { 0x06, 0x03 },
    { 0x17, 0x1d },
    { 0x76, 0x00 },
    { 0x72, 0x3e },
    { 0x84, 0x00 },
    { 0xe8, 0xe2 },
    { 0x8a, 0x03 },
    { 0x96, 0xef },
    { 0x76, 0x00 },
	{ CFG_META_DELAY, 100 },
    { CFG_META_BURST, 21 },
    { 0x34, 0x00 },
    { 0x79, 0x22 },
    { 0x7d, 0x03 },
    { 0x2a, 0x5d },
    { 0xe5, 0x00 },
    { 0x67, 0x7e },
    { 0x17, 0x00 },
    { 0xd5, 0xa2 },
    { 0x1b, 0x03 },
    { 0x9f, 0x5f },
    { 0x6b, 0x00 },
	{ CFG_META_DELAY, 100 },
    { CFG_META_BURST, 21 },
    { 0x35, 0x00 },
    { 0x79, 0x42 },
    { 0x12, 0x03 },
    { 0x17, 0x61 },
    { 0xe0, 0x00 },
    { 0x70, 0x56 },
    { 0xed, 0x00 },
    { 0xe8, 0x9e },
    { 0x20, 0x03 },
    { 0x96, 0x67 },
    { 0x01, 0x00 },
	{ CFG_META_DELAY, 100 },
    { CFG_META_BURST, 21 },
    { 0x36, 0x00 },
    { 0x7c, 0x61 },
    { 0x61, 0x03 },
    { 0x33, 0x49 },
    { 0x22, 0x00 },
    { 0x70, 0x00 },
    { 0xdc, 0x00 },
    { 0xcc, 0xb6 },
    { 0xde, 0x03 },
    { 0x93, 0x9d },
    { 0xc3, 0x00 },
	{ CFG_META_DELAY, 100 },
    { CFG_META_BURST, 21 },
    { 0x37, 0x00 },
    { 0x76, 0xf2 },
    { 0xe1, 0x03 },
    { 0x25, 0x01 },
    { 0xb9, 0x00 },
    { 0x68, 0x55 },
    { 0xba, 0x00 },
    { 0xda, 0xfe },
    { 0x47, 0x03 },
    { 0xa0, 0xb7 },
    { 0x66, 0x00 },
	{ CFG_META_DELAY, 100 },
    { CFG_META_BURST, 21 },
    { 0x38, 0x00 },
    { 0x80, 0x00 },
    { 0x00, 0x00 },
    { 0x00, 0x00 },
    { 0x00, 0x00 },
    { 0x00, 0x00 },
    { 0x00, 0x00 },
    { 0x00, 0x00 },
    { 0x00, 0x00 },
    { 0x00, 0x00 },
    { 0x00, 0x00 },
	{ CFG_META_DELAY, 100 },
    { CFG_META_BURST, 21 },
    { 0x39, 0x00 },
    { 0x4d, 0xe0 },
    { 0xc7, 0x03 },
    { 0x94, 0xe7 },
    { 0x19, 0x00 },
    { 0x29, 0x26 },
    { 0x6f, 0x00 },
    { 0xc3, 0x17 },
    { 0x77, 0x03 },
    { 0xb0, 0xfa },
    { 0x3a, 0x00 },
	{ CFG_META_DELAY, 100 },
    { CFG_META_BURST, 9 },
    { 0x3b, 0x00 },
    { 0x07, 0x60 },
    { 0x53, 0x00 },
    { 0x78, 0x9f },
    { 0xac, 0x00 },
    { CFG_META_BURST, 9 },
    { 0x3c, 0x00 },
    { 0x00, 0x00 },
    { 0x6d, 0xff },
    { 0xff, 0xff },
    { 0xe5, 0x00 },
    { CFG_META_BURST, 9 },
    { 0x3e, 0x00 },
    { 0x07, 0x60 },
    { 0x53, 0x00 },
    { 0x78, 0x9f },
    { 0xac, 0x00 },
    { CFG_META_BURST, 9 },
    { 0x3f, 0x00 },
    { 0x00, 0x00 },
    { 0x6d, 0xff },
    { 0xff, 0xff },
    { 0xe5, 0x00 },
    { CFG_META_BURST, 5 },
    { 0x40, 0x00 },
    { 0xcc, 0xcc },
    { 0xcc, 0x00 },
    { CFG_META_BURST, 5 },
    { 0x41, 0x08 },
    { 0x00, 0x00 },
    { 0x00, 0x00 },
    { CFG_META_BURST, 9 },
    { 0x42, 0x00 },
    { 0x00, 0x00 },
    { 0x6d, 0xff },
    { 0xff, 0xff },
    { 0xe5, 0x00 },
    { CFG_META_BURST, 5 },
    { 0x43, 0x08 },
    { 0x00, 0x00 },
    { 0x00, 0x00 },
    { CFG_META_BURST, 5 },
    { 0x44, 0x07 },
    { 0x21, 0x48 },
    { 0x2b, 0x00 },
    { CFG_META_BURST, 9 },
    { 0x45, 0x00 },
    { 0x00, 0x00 },
    { 0x6d, 0xff },
    { 0xff, 0xff },
    { 0xe5, 0x00 },
    { CFG_META_BURST, 5 },
    { 0x46, 0x00 },
    { 0x02, 0x00 },
    { 0x09, 0x00 },
    { CFG_META_BURST, 9 },
    { 0x47, 0x00 },
    { 0x07, 0x60 },
    { 0x53, 0x00 },
    { 0x78, 0x9f },
    { 0xac, 0x00 },
    { CFG_META_BURST, 9 },
    { 0x48, 0x00 },
    { 0x07, 0x60 },
    { 0x53, 0x00 },
    { 0x78, 0x9f },
    { 0xac, 0x00 },
    { CFG_META_BURST, 5 },
    { 0x4f, 0x00 },
    { 0x00, 0x00 },
    { 0x08, 0x00 },
    { CFG_META_BURST, 5 },
    { 0x50, 0x0f },
    { 0x70, 0x80 },
    { 0x00, 0x00 },
    { CFG_META_BURST, 13 },
    { 0x51, 0x00 },
    { 0x80, 0x00 },
    { 0x00, 0x00 },
    { 0x80, 0x00 },
    { 0x00, 0x03 },
    { 0x80, 0x00 },
    { 0x00, 0x00 },
    { CFG_META_BURST, 13 },
    { 0x52, 0x00 },
    { 0x80, 0x00 },
    { 0x00, 0x00 },
    { 0x80, 0x00 },
    { 0x00, 0x03 },
    { 0x80, 0x00 },
    { 0x00, 0x00 },
    { CFG_META_BURST, 5 },
    { 0x56, 0x00 },
    { 0x80, 0x00 },
    { 0x00, 0x00 },
    { CFG_META_BURST, 5 },
    { 0x57, 0x00 },
    { 0x02, 0x00 },
    { 0x00, 0x00 },
    { CFG_META_BURST, 21 },
    { 0x58, 0x00 },
    { 0x5d, 0xf3 },
    { 0x52, 0x00 },
    { 0x1e, 0xec },
    { 0x51, 0x00 },
    { 0x19, 0x86 },
    { 0xf9, 0x03 },
    { 0xe1, 0x13 },
    { 0xaf, 0x00 },
    { 0x08, 0x85 },
    { 0xb5, 0x00 },
	{ CFG_META_DELAY, 100 },
    { CFG_META_BURST, 21 },
    { 0x59, 0x00 },
    { 0x00, 0x03 },
    { 0x19, 0x00 },
    { 0x00, 0x06 },
    { 0x32, 0x00 },
    { 0x00, 0x03 },
    { 0x19, 0x00 },
    { 0xfb, 0x05 },
    { 0xac, 0x03 },
    { 0x84, 0xed },
    { 0xf0, 0x00 },
	{ CFG_META_DELAY, 100 },
    { CFG_META_BURST, 21 },
    { 0x5a, 0x00 },
    { 0x59, 0x0c },
    { 0xdc, 0x03 },
    { 0x4d, 0xe6 },
    { 0x47, 0x00 },
    { 0x59, 0x0c },
    { 0xdc, 0x00 },
    { 0xab, 0x0d },
    { 0xc1, 0x03 },
    { 0xc6, 0xda },
    { 0x4f, 0x00 },
    { CFG_META_BURST, 21 },
    { 0x5b, 0x00 },
    { 0x03, 0x85 },
    { 0xfc, 0x00 },
    { 0x07, 0x0b },
    { 0xf8, 0x00 },
    { 0x03, 0x85 },
    { 0xfc, 0x00 },
    { 0xab, 0x0d },
    { 0xc1, 0x03 },
    { 0xc6, 0xda },
    { 0x4f, 0x00 },
    { CFG_META_BURST, 21 },
    { 0x5c, 0x00 },
    { 0x7d, 0x85 },
    { 0xef, 0x03 },
    { 0x04, 0xf4 },
    { 0x22, 0x00 },
    { 0x7d, 0x85 },
    { 0xef, 0x00 },
    { 0xfb, 0x05 },
    { 0xac, 0x03 },
    { 0x84, 0xed },
    { 0xf0, 0x00 },
	{ CFG_META_DELAY, 100 },
    { CFG_META_BURST, 21 },
    { 0x5d, 0x00 },
    { 0x5d, 0xf3 },
    { 0x52, 0x00 },
    { 0x1e, 0xec },
    { 0x51, 0x00 },
    { 0x19, 0x86 },
    { 0xf9, 0x03 },
    { 0xe1, 0x13 },
    { 0xaf, 0x00 },
    { 0x08, 0x85 },
    { 0xb5, 0x00 },
	{ CFG_META_DELAY, 100 },
    { CFG_META_BURST, 21 },
    { 0x5e, 0x00 },
    { 0x00, 0x03 },
    { 0x19, 0x00 },
    { 0x00, 0x06 },
    { 0x32, 0x00 },
    { 0x00, 0x03 },
    { 0x19, 0x00 },
    { 0xfb, 0x05 },
    { 0xac, 0x03 },
    { 0x84, 0xed },
    { 0xf0, 0x00 },
	{ CFG_META_DELAY, 100 },
    { CFG_META_BURST, 21 },
    { 0x5f, 0x00 },
    { 0x59, 0x0c },
    { 0xdc, 0x03 },
    { 0x4d, 0xe6 },
    { 0x47, 0x00 },
    { 0x59, 0x0c },
    { 0xdc, 0x00 },
    { 0xab, 0x0d },
    { 0xc1, 0x03 },
    { 0xc6, 0xda },
    { 0x4f, 0x00 },
    { CFG_META_BURST, 21 },
    { 0x60, 0x00 },
    { 0x03, 0x85 },
    { 0xfc, 0x00 },
    { 0x07, 0x0b },
    { 0xf8, 0x00 },
    { 0x03, 0x85 },
    { 0xfc, 0x00 },
    { 0xab, 0x0d },
    { 0xc1, 0x03 },
    { 0xc6, 0xda },
    { 0x4f, 0x00 },
    { CFG_META_BURST, 21 },
    { 0x61, 0x00 },
    { 0x7d, 0x85 },
    { 0xef, 0x03 },
    { 0x04, 0xf4 },
    { 0x22, 0x00 },
    { 0x7d, 0x85 },
    { 0xef, 0x00 },
    { 0xfb, 0x05 },
    { 0xac, 0x03 },
    { 0x84, 0xed },
    { 0xf0, 0x00 },
	{ CFG_META_DELAY, 100 },
    { CFG_META_BURST, 5 },
    { 0x70, 0x00 },
    { 0x80, 0x00 },
    { 0x00, 0x00 },
    { CFG_META_BURST, 5 },
    { 0x71, 0x00 },
    { 0x00, 0x00 },
    { 0x00, 0x00 },
    { CFG_META_BURST, 5 },
    { 0x72, 0x00 },
    { 0x00, 0x00 },
    { 0x00, 0x00 },
    { CFG_META_BURST, 5 },
    { 0x73, 0x00 },
    { 0x80, 0x00 },
    { 0x00, 0x00 },
    { CFG_META_BURST, 5 },
    { 0x74, 0x00 },
    { 0x80, 0x00 },
    { 0x00, 0x00 },
    { CFG_META_BURST, 5 },
    { 0x75, 0x00 },
    { 0x00, 0x00 },
    { 0x00, 0x00 },
    { CFG_META_BURST, 5 },
    { 0x76, 0x00 },
    { 0x00, 0x00 },
    { 0x00, 0x00 },
    { CFG_META_BURST, 5 },
    { 0x77, 0x00 },
    { 0x80, 0x00 },
    { 0x00, 0x00 }
};

